SystemOrganization addCategory: #'Pier-FuelPersistence-Core'!

PRDelayedPersistency subclass: #PRFuelPersistency
	instanceVariableNames: 'directory directoryName namePrefix maxImages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FuelPersistence-Core'!

PRFuelPersistency class
	instanceVariableNames: 'defaultDirectoryName'!

!PRFuelPersistency commentStamp: 'YanniChiu 5/18/2012 01:06' prior: 0!
A Fuel serialization based persistency mechanism.

To enable:
- login to Pier as admin
- navigate to system/management/persistency
- change persistency to "Fuel Persistency"
- make a persistency snapshot
- save the smalltalk image

Afterwards, on image startup, just before the Seaside server startup, the PRFuelPersistency startup will restore any saved kernel.!

PRFuelPersistency class
	instanceVariableNames: 'defaultDirectoryName'!

!PRFuelPersistency class methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
defaultDirectoryName
	defaultDirectoryName isNil ifTrue: [ defaultDirectoryName := 'snapshot' ].
	^ defaultDirectoryName! !

!PRFuelPersistency class methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
defaultDirectoryName: anObject
	defaultDirectoryName := anObject! !

!PRFuelPersistency class methodsFor: 'accessing-descriptions' stamp: 'YanniChiu 5/31/2011 23:38'!
descriptionDirectoryName
	^ MAStringDescription new
		comment: 'The directory path for snapshot files.';
		label: 'Snapshot Directory Name';
		accessor: #directoryName;
		default: nil;
		priority: 120;
		yourself! !

!PRFuelPersistency class methodsFor: 'accessing-descriptions' stamp: 'YanniChiu 5/31/2011 23:38'!
descriptionMaxImages
	^ MANumberDescription new
		comment: 'The maximum number of backup copies to keep';
		accessor: #maxImages;
		label: 'Backup Images';
		priority: 140;
		beRequired;
		default: 3;
		beInteger;
		yourself! !

!PRFuelPersistency class methodsFor: 'accessing-descriptions' stamp: 'YanniChiu 5/31/2011 23:38'!
descriptionNamePrefix
	^ MAStringDescription new
		comment: 'The prefix for snapshot file names.';
		label: 'Snapshot Prefix';
		accessor: #namePrefix;
		default: nil;
		priority: 130;
		yourself! !

!PRFuelPersistency class methodsFor: 'as yet unclassified' stamp: 'YanniChiu 6/14/2011 12:02'!
fuelIgnoredInstanceVariableNames
    ^#(directory)! !

!PRFuelPersistency class methodsFor: 'importing' stamp: 'YanniChiu 5/31/2011 23:43'!
importFromDirectoryReference: reference withPrefix: aString
	| persistency kernel |
	persistency := PRFuelPersistency new directory: reference; namePrefix: aString; yourself.
	kernel := persistency deserialize.
	^kernel
! !

!PRFuelPersistency class methodsFor: 'importing' stamp: 'YanniChiu 8/14/2011 21:35'!
importFromPathString: aPathString withPrefix: aString
	| directory |
	directory := (FSFilesystem disk resolveString: aPathString) asReference.
	^ self importFromDirectoryReference: directory withPrefix: aString
! !

!PRFuelPersistency class methodsFor: 'class initialization' stamp: 'YanniChiu 5/18/2012 01:05'!
initialize
	Smalltalk addToStartUpList: self before: WAServerManager! !

!PRFuelPersistency class methodsFor: 'system startup' stamp: 'MarianoMartinezPeck 5/18/2012 12:07'!
restoreKernel
	"Check the registered applications for Pier apps that use
	PRFuelPersistency, and if found, then restore the saved kernel."

	"Note that only the first level children are examined below,"
	"so make sure the pier application is register at the Seaside root."
	WADispatcher default childrenDo: [ :app | | rootClass oldKernel kernel |
		[ rootClass := app preferenceAt: #rootClass ]
			on: WAAttributeNotFound do: [ rootClass := nil ].
		(rootClass notNil and: [rootClass includesBehavior: PRPierFrame]) ifTrue: [
			oldKernel := app preferenceAt: #kernel.
			(oldKernel notNil and: [ oldKernel persistency isKindOf: PRFuelPersistency ]) ifTrue: [
				PRFuelExporterImporter new
					importAndReplaceOldKernel: oldKernel 
					from: oldKernel persistency materializationStream 
					inApp: app. 
			]
		]
	]! !

!PRFuelPersistency class methodsFor: 'system startup' stamp: 'YanniChiu 5/18/2012 00:52'!
startUp
	self restoreKernel! !

!PRFuelPersistency class methodsFor: 'class initialization' stamp: 'YanniChiu 5/16/2012 16:18'!
unload
	GRPlatform current removeFromStartUpList: self! !

!PRFuelPersistency methodsFor: 'actions' stamp: 'YanniChiu 5/31/2011 23:38'!
cleanupBackupImages
	| entries |
	maxImages isNil ifTrue: [ ^ self ].
	entries := self directory entries select: [ :each | each isFile and: [ each basename beginsWith: self namePrefix ] ].
	entries size <= maxImages ifTrue: [ ^ self ].
	entries := (SortedCollection sortBlock: [ :a :b | a basename < b basename ]) addAll: entries; yourself.
	entries := entries allButLast: maxImages.
	entries do: [ :each | each reference delete ]! !

!PRFuelPersistency methodsFor: 'actions' stamp: 'YanniChiu 5/31/2011 23:38'!
desanitize
	super desanitize.
	self directory: nil.
! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/15/2012 12:45'!
directory
	directory isNil ifTrue: [
		directory := FileSystem disk referenceTo: self directoryName.
		FileSystem disk ensureDirectory: directory.
	].
	^ directory! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
directory: anObject
	directory := anObject! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
directoryName
	"Answer the directoryName value. Use the default value if nil, but do not update the inst. var. to the default value;
	otherwise, it will be persisted, and thus prevent changes during deployment."
	^ (directoryName isNil or: [ directoryName isEmpty ])
		ifTrue: [ self class defaultDirectoryName ]
		ifFalse: [ directoryName ]! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
directoryName: anObject
	directoryName := anObject! !

!PRFuelPersistency methodsFor: 'private' stamp: 'YanniChiu 5/31/2011 23:38'!
log: aContext
	| ref stream |
	ref := self directory / 'transactions.txt'.
	stream := ref writeStream.
	[
		stream position: ref entry size + 1.
		self log: aContext on: stream
	]
	ensure: [ stream close ]! !

!PRFuelPersistency methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/18/2012 12:07'!
materializationStream 
	| ref stream |
	ref := self newestSnapshotIn: self directory.
	stream := MultiByteBinaryOrTextStream with: ref readStream contents.
	stream binary.
	stream converter: TextConverter default.
	stream reset.
	^ stream! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
maxImages
	^ maxImages! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
maxImages: anObject
	maxImages := anObject! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
namePrefix
	"Answer the namePrefix value. Use the default value if nil, but do not update the inst. var. to the default value;
	otherwise, it will be persisted, and thus prevent changes during deployment."
	^ (namePrefix isNil or: [ namePrefix isEmpty ])
		ifTrue: [ self kernel name, '_' ]
		ifFalse: [ namePrefix ]! !

!PRFuelPersistency methodsFor: 'accessing' stamp: 'YanniChiu 5/31/2011 23:38'!
namePrefix: aString
	namePrefix := aString! !

!PRFuelPersistency methodsFor: 'private' stamp: 'YanniChiu 5/31/2011 23:38'!
newestSnapshotIn: directoryReference
	| entries |
	entries := directoryReference entries select: [ :each | each isFile and: [ each basename beginsWith: self namePrefix ] ].
	entries := (SortedCollection sortBlock: [ :a :b | a basename >= b basename ]) addAll: entries; yourself.
	entries size > 0
		ifTrue: [ ^ entries first reference ]
		ifFalse: [ self error: 'No snapshots found in: ', self directory greaseString ]! !

!PRFuelPersistency methodsFor: 'actions' stamp: 'YanniChiu 5/31/2011 23:38'!
snapshot
	| snapshotName |
	self snapshotWithName: (snapshotName := self snapshotName).
	self message: 'Last snapshot ' , snapshotName printString , ' at ' , GRPlatform current magritteTimeStampClass now greaseString.
	self cleanupBackupImages! !

!PRFuelPersistency methodsFor: 'private' stamp: 'YanniChiu 5/31/2011 23:38'!
snapshotName
	^ self snapshotNameFromDateAndTime: DateAndTime now! !

!PRFuelPersistency methodsFor: 'actions' stamp: 'MarianoMartinezPeck 5/15/2012 12:52'!
snapshotNameFromDateAndTime: dateAndTime
	^ self namePrefix , dateAndTime asSortableString , '.fuel'! !

!PRFuelPersistency methodsFor: 'actions' stamp: 'MarianoMartinezPeck 5/18/2012 11:55'!
snapshotWithName: filename
	[ | stream | 
	stream := (self directory / filename) writeStream.
	self sanitize.
	PRFuelExporterImporter new exportKernel: self kernel to: stream.
	self desanitize.
	stream close ] valueUnpreemptively! !

Object subclass: #PRFuelExporterImporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-FuelPersistence-Core'!

!PRFuelExporterImporter methodsFor: 'exporting' stamp: 'MarianoMartinezPeck 5/18/2012 11:54'!
exportKernel: aKernel to: aStream
	FLSerializer newDefault serialize: aKernel on: aStream.
! !

!PRFuelExporterImporter methodsFor: 'exporting' stamp: 'MarianoMartinezPeck 5/18/2012 12:18'!
exportKernel: aKernel toFilename: aFilename
	FLSerializer newDefault serialize: aKernel toFileNamed: aFilename.
! !

!PRFuelExporterImporter methodsFor: 'exporting' stamp: 'MarianoMartinezPeck 5/18/2012 13:02'!
exportKernelNamed: aKernelName toFilename: aFilename
	FLSerializer newDefault serialize: (PRKernel instanceNamed: aKernelName) toFileNamed: aFilename.
! !

!PRFuelExporterImporter methodsFor: 'importing' stamp: 'MarianoMartinezPeck 5/18/2012 12:06'!
importAndReplaceOldKernel: oldKernel from: stream inApp: app
	| newKernel |
	PRKernel instances remove: oldKernel ifAbsent: [ nil ].
	newKernel := (FLMaterializer newDefault materializeFrom: stream) root.
	newKernel persistency desanitize.
	PRKernel named: newKernel name root: newKernel root.
	app preferenceAt: #kernel put: newKernel.! !

!PRFuelExporterImporter methodsFor: 'importing' stamp: 'MarianoMartinezPeck 5/18/2012 13:07'!
importNewKernelFrom: aStream
	| kernel application |
	kernel := self materializeKernelFrom: aStream.
	^ self registerKernel: kernel withName: kernel name.
	
! !

!PRFuelExporterImporter methodsFor: 'importing' stamp: 'MarianoMartinezPeck 5/18/2012 13:20'!
importNewKernelFromFilename: aFilename
	| kernel application |
	FileStream oldFileNamed: aFilename do: [:aStream |
		aStream binary.
		kernel := self materializeKernelFrom: aStream.
		^ self registerKernel: kernel withName: kernel name.
		] 
	! !

!PRFuelExporterImporter methodsFor: 'importing' stamp: 'MarianoMartinezPeck 5/18/2012 13:06'!
importNewKernelNamed: kernelName from: aStream
	| kernel application |
	kernel := self materializeKernelFrom: aStream.
	^ self registerKernel: kernel withName: kernelName.
	! !

!PRFuelExporterImporter methodsFor: 'importing' stamp: 'MarianoMartinezPeck 5/18/2012 12:35'!
importNewKernelNamed: kernelName fromFilename: aFilename
	FileStream oldFileNamed: aFilename do: [:aStream |
		aStream binary.
		^ self importNewKernelNamed: kernelName from: aStream
		] 
	! !

!PRFuelExporterImporter methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/18/2012 13:05'!
materializeKernelFrom: aStream
	| kernel |
	kernel := (FLMaterializer newDefault materializeFrom: aStream) root. 	
	(kernel isKindOf: PRKernel)
		ifFalse: [ ^ self error: 'Invalid file format.' ].
	^ kernel! !

!PRFuelExporterImporter methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/18/2012 13:15'!
registerKernel: kernel withName: kernelName
	| application |
	kernel name: kernelName.
	application := PRPierFrame
		registerAsApplication: kernelName
		kernel: kernel.
	application parent defaultName: kernelName.
	WADispatcher default defaultName: kernelName.
	PRKernel instances add: kernel.
	^ kernel

	! !

!MADynamicObject methodsFor: '*Pier-FuelPersistence-Core' stamp: 'MarianoMartinezPeck 5/14/2012 14:46'!
fuelAccept: aVisitor  
	^aVisitor visitSubstitution: self by: nil! !
PRFuelPersistency initialize!
